// Copyright (C) 2013 The Android Open Source Project
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.gerrit.client.ui;

import com.google.gerrit.client.Gerrit;
import com.google.gerrit.client.changes.ChangeDetailCache;
import com.google.gerrit.common.PageLinks;
import com.google.gerrit.common.data.ChangeDetail;
import com.google.gwt.user.client.ui.FocusWidget;

public abstract class ActionDialog extends CommentedActionDialog<ChangeDetail> {
  public ActionDialog(final FocusWidget enableOnFailure, final boolean redirect,
      String dialogTitle, String dialogHeading) {
    super(dialogTitle, dialogHeading, new ChangeDetailCache.IgnoreErrorCallback() {
        @Override
        public void onSuccess(ChangeDetail result) {
          if (redirect) {
            Gerrit.display(PageLinks.toChange(result.getChange().getId()));
          } else {
            super.onSuccess(result);
          }
        }

        @Override
        public void onFailure(Throwable caught) {
          enableOnFailure.setEnabled(true);
        }
      });
  }
}